#include "3dviewer.h"



C3D_Viewer::C3D_Viewer()
{
    initialized = false;
};

C3D_Viewer::~C3D_Viewer()
{
    if (initialized)
	XDestroyWindow(display, win);
};

void C3D_Viewer::getxy(
		       int a,
		       int b,
		       float h,
		       
		       float eye_a,
		       float eye_b,
		       float eye_h,
		       float angle,
		       
		       int &x,
		       int &y)
{
    float dist = sqrt(pow(a-eye_a, 2) + pow(b-eye_b, 2));
    float da =  (a-eye_a)*cos(angle) + (b-eye_b)*sin(angle);
    float db = -(a-eye_a)*sin(angle) + (b-eye_b)*cos(angle);

    y = horizonty + (int)((augen_y * (eye_h-h))/(db));
    x = 500 + (int)((augen_x * (da))/(dist));
};

void C3D_Viewer::viereck(
			 int a1, int b1, float h1,
			 int a2, int b2, float h2,
			 int a3, int b3, float h3,
			 int a4, int b4, float h4)
{
    int x1,y1;
    int x2,y2;
    int x3,y3;
    int x4,y4;

 
    getxy( a1, b1, h1,  me_a ,me_b, me_h, me_an,  x1, y1);
    getxy( a2, b2, h2,  me_a ,me_b, me_h, me_an,  x2, y2);
    getxy( a3, b3, h3,  me_a ,me_b, me_h, me_an,  x3, y3);
    getxy( a4, b4, h4,  me_a ,me_b, me_h, me_an,  x4, y4);

    XDrawLine(display, win, gc1, x1, y1, x2, y2);
    XDrawLine(display, win, gc1, x2, y2, x3, y3);
    XDrawLine(display, win, gc1, x3, y3, x4, y4);
    XDrawLine(display, win, gc1, x4, y4, x1, y1);
};

void C3D_Viewer::wuerfel(
			 int a1, int b1, int h1,
			 int a2, int b2, int h2,
			 int a3, int b3, int h3,
			 int a4, int b4, int h4,
			 int a5, int b5, int h5,
			 int a6, int b6, int h6,
			 int a7, int b7, int h7,
			 int a8, int b8, int h8)
{
    int x1,y1;
    int x2,y2;
    int x3,y3;
    int x4,y4;
    int x5,y5;
    int x6,y6;
    int x7,y7;
    int x8,y8;

 
    getxy( a1, b1, h1,  me_a ,me_b, me_h, me_an,  x1, y1);
    getxy( a2, b2, h2,  me_a ,me_b, me_h, me_an,  x2, y2);
    getxy( a3, b3, h3,  me_a ,me_b, me_h, me_an,  x3, y3);
    getxy( a4, b4, h4,  me_a ,me_b, me_h, me_an,  x4, y4);
    getxy( a5, b5, h5,  me_a ,me_b, me_h, me_an,  x5, y5);
    getxy( a6, b6, h6,  me_a ,me_b, me_h, me_an,  x6, y6);
    getxy( a7, b7, h7,  me_a ,me_b, me_h, me_an,  x7, y7);
    getxy( a8, b8, h8,  me_a ,me_b, me_h, me_an,  x8, y8);

    XDrawLine(display, win, gc1, x1, y1, x2, y2);
    XDrawLine(display, win, gc1, x2, y2, x3, y3);
    XDrawLine(display, win, gc1, x3, y3, x4, y4);
    XDrawLine(display, win, gc1, x4, y4, x1, y1);

    XDrawLine(display, win, gc1, x1, y1, x5, y5);
    XDrawLine(display, win, gc1, x2, y2, x6, y6);
    XDrawLine(display, win, gc1, x3, y3, x7, y7);
    XDrawLine(display, win, gc1, x4, y4, x8, y8);

    XDrawLine(display, win, gc1, x5, y5, x6, y6);
    XDrawLine(display, win, gc1, x6, y6, x7, y7);
    XDrawLine(display, win, gc1, x7, y7, x8, y8);
    XDrawLine(display, win, gc1, x8, y8, x5, y5);
};

void C3D_Viewer::init_windows()
{
    /* Attempt to connect to the server and use the default screen */
    if ((display = XOpenDisplay(NULL)) == NULL) {
	perror("Can't connect to server");
	exit(1);
    }
    screen = DefaultScreen(display);
    
    /* Create a window */
    win = XCreateSimpleWindow(display, RootWindow(display, screen),
			      100, 200, 1000, 800, 1, BlackPixel(display, screen),
			      WhitePixel(display, screen));
    
    initialized= true;
};

void C3D_Viewer::show(CGrowing_Grid &growing_grid)
{

    XKeyEvent *kev;
    
    if (!initialized)
	init_windows();
    
    /* Set up which event types the window will handle */
    XSelectInput(display, win, ExposureMask | ButtonPressMask | KeyPressMask);
    
    /* Graphics contexts */
    setUpGCs();
    
    /* Attempt to get message from user-defined resource setting */


//    if ((message = XGetDefault(display, "xlib_demo", "message")) == NULL)
//	message = progDefaultMessage;




    
    /* Map the window in order for it to be displayed */
    XMapWindow(display, win);
    
    
    drawInToWindow(growing_grid);
    
    /* Enter an infinite loop, awaiting and responding to the chosen events */
    while (1) {
	XNextEvent(display, &event);
	switch (event.type) {
	case Expose: drawInToWindow(growing_grid); break; /* Draw/re-draw everything */
	case ButtonPress: return; break; /* Quit on mouse button press */
	case KeyPress: 
	    kev = (XKeyEvent*)&event;
	    if (kev->keycode==38) level = (level < 63) ? level+1 : level;// a
	    if (kev->keycode==52) level = (level > 0 ) ? level-1 : level;// z
	    
	    if (kev->keycode==39) subs = (subs < 7) ? subs+1 : subs;// s
	    if (kev->keycode==53) subs = (subs > 0) ? subs-1 : subs;// x


	    if (kev->keycode==83) me_a-=5.0;// 4
	    if (kev->keycode==85) me_a+=5.0;// 6

	    if (kev->keycode==80) me_b+=5.0;// 8
	    if (kev->keycode==88) me_b-=5.0;// 2
	
	    if (kev->keycode==90) me_an += 0.05;// 0
	    if (kev->keycode==91) me_an -= 0.05;// .

	    if (kev->keycode==86 ) me_h += delta_h;// +
	    if (kev->keycode==108) me_h -= delta_h;// <cr>

	    print_status();

	    drawInToWindow(growing_grid);
	    break;
	default: break;
	};
    };
};

void C3D_Viewer::setUpGCs()
    /* Set up 2 graphics context resources in the display server */
    /* gc1 for message and rectangle; gc2 for ellipse */
{
    /* Define a bitmap for the stipple pattern used to fill the ellipse */
#define stipple_width 3
#define stipple_height 3
    static char stipple_bits[] = {0x02, 0x05, 0x02};
    Pixmap stipple;
    
    /* Create the stipple for ellipse fill pattern of gc2 */
    if ((stipple = XCreateBitmapFromData(display, RootWindow(display, screen),
					 stipple_bits, stipple_width, stipple_height)) == 0) {
	perror("Can't create bitmap");
	exit(1);
    }
    
    gc1 = XCreateGC(display, win, 0, &gs1); /* 0 -> use defaults */
    gc2 = XCreateGC(display, win, 0, &gs2); /* 0 -> use defaults */
    /* Now override some of the defaults */
    XSetForeground(display, gc1, BlackPixel(display, screen));
    XSetBackground(display, gc1, WhitePixel(display, screen));
    XSetLineAttributes(display, gc1, 1, LineSolid, CapRound, JoinRound);
    XSetStipple(display, gc2, stipple);
    XSetFillStyle(display, gc2, FillSolid);
};






















C3D_Viewer_Occupied::C3D_Viewer_Occupied():C3D_Viewer()
{
    delta_h   = 5;
    augen_x   = 800;
    augen_y   = 800;
    horizonty = 500;


    level = 40;

    me_a = 32;
    me_b = -75;
    me_h = 25;
    me_an = 0;

};

C3D_Viewer_Occupied::~C3D_Viewer_Occupied()
{
    // nothing to be done
};

void C3D_Viewer_Occupied::print_status()
{
    printf("Level: %d, a: %f, b: %f, h: %f, a: %f\n", level, me_a, me_b, me_h, me_an);
};

void C3D_Viewer_Occupied::drawInToWindow(CGrowing_Grid &growing_grid)
{
  XSetForeground(display, gc1, WhitePixel(display, screen));

  for (int y= 0 ; y<799; y++)
    XDrawLine(display, win, gc1, 0,y,999,y);

  XSetForeground(display, gc1, BlackPixel(display, screen));

  for (int a = 0; a<64; a++)
    for (int b = 0; b<64; b++)
      for (int c = 0; c<64; c++)
	if (growing_grid.get_occupied(a,b,c))
	    wuerfel(a  ,b  ,c  ,
		    a+1,b  ,c  ,
		    a+1,b+1,c  ,
		    a  ,b+1,c  ,
		    a  ,b  ,c+1,
		    a+1,b  ,c+1,
		    a+1,b+1,c+1,
		    a  ,b+1,c+1);
  wuerfel(  0, 0, 0,
	   64, 0, 0,
	   64,64, 0,
	    0,64, 0,
	    0, 0,64,
	   64, 0,64,
	   64,64,64,
	    0,64,64);
};






















C3D_Viewer_Concentration::C3D_Viewer_Concentration():C3D_Viewer()
{
    subs      = 7;
    delta_h   = 0.05;
    augen_x   = 800;
    augen_y   = 100000;
    horizonty = 100;

    level = 40;

    me_a = 32;
    me_b = -50;
    me_h = 0.33;
    me_an = 0;

};

C3D_Viewer_Concentration::~C3D_Viewer_Concentration()
{
    // nothing to be done
};


void C3D_Viewer_Concentration::print_status()
{
    printf("Level: %d, Substance: %d;    a: %f, b: %f, h: %f, a: %f\n", level, subs,  me_a, me_b, me_h, me_an);
};


void C3D_Viewer_Concentration::drawInToWindow(CGrowing_Grid &growing_grid)
{
  XSetForeground(display, gc1, WhitePixel(display, screen));

  for (int y= 0 ; y<799; y++)
    XDrawLine(display, win, gc1, 0,y,999,y);

  XSetForeground(display, gc1, BlackPixel(display, screen));

  for (int a = 0; a<63; a++)
    for (int b = 0; b<63; b++)
	viereck(a  ,b  , growing_grid.get_concentration(a  ,b  ,level, subs), 
		a+1,b  , growing_grid.get_concentration(a+1,b  ,level, subs), 
		a+1,b+1, growing_grid.get_concentration(a+1,b+1,level, subs), 
		a  ,b+1, growing_grid.get_concentration(a  ,b+1,level, subs)
		);
//  XSetForeground(display, gc1, 123514);

//   viereck(0  ,0 , 0.0,
// 	  64 ,0 , 0.0,
// 	  64 ,64, 0.0,
// 	  0  ,64, 0.0
// 	  );

//   viereck(0  ,0 , 0.2,
// 	  64 ,0 , 0.2,
// 	  64 ,64, 0.2,
// 	  0  ,64, 0.2
// 	  );

//   viereck(0  ,0 , 0.4,
// 	  64 ,0 , 0.4,
// 	  64 ,64, 0.4,
// 	  0  ,64, 0.4
// 	  );

//   viereck(0  ,0 , 0.6,
// 	  64 ,0 , 0.6,
// 	  64 ,64, 0.6,
// 	  0  ,64, 0.6
// 	  );

//   viereck(0  ,0 , 0.8,
// 	  64 ,0 , 0.8,
// 	  64 ,64, 0.8,
// 	  0  ,64, 0.8
// 	  );

//   viereck(0  ,0 , 1.0,
// 	  64 ,0 , 1.0,
// 	  64 ,64, 1.0,
// 	  0  ,64, 1.0
// 	  );
};
