#include "evolutionmaster.h"
#include <math.h>
#include <BitString.h>
#include <assert.h>

CEvolution_Master::CEvolution_Master()
{
    char str[GENOTYPE_LENGTH_WITH_TERM];
    str[GENOTYPE_LENGTH_WITH_TERM-1] = '\0';
    
    for (int i=0; i<POPULATION_SIZE; i++)
	{
	    for (int j=0; j<GENOTYPE_LENGTH; j++)
		{
		    str[j] = (char)(drand48()*2.0) + '0';
	      };
	    genepool[i] = new MyBitString ( atoMyBitString(str) );
	};
    
    world=new CWorld(MAP_FILE);
};

CEvolution_Master::CEvolution_Master(FILE *in_file)
{
    char str[GENOTYPE_LENGTH_WITH_TERM];
    str[GENOTYPE_LENGTH_WITH_TERM-1] = '\0';
    
    for (int i=0; i<POPULATION_SIZE; i++)
	{
	    for (int j=0; j<GENOTYPE_LENGTH; j++)
		{
		    str[j] = getc(in_file);
	      };
	    genepool[i] = new MyBitString ( atoMyBitString(str) );
	    getc(in_file);
	};

    world=new CWorld(MAP_FILE);
};

CEvolution_Master::~CEvolution_Master()
{
    for (int i=0; i<POPULATION_SIZE; i++)
	delete genepool[i];
    delete world;
};

void CEvolution_Master::make_turn()
{
    calc_fitnesses();
    reproduce();
};

void CEvolution_Master::calc_fitnesses()
{
    world->evaluate(genepool);
};

void CEvolution_Master::Crossover(
				  BitString &creature1,
				  BitString &creature2
				  )
{
    if (drand48() < REPRODUCE_PROBABILITY_CROSSOVER)
	{
	    int pos = (int)(drand48()*(GENOTYPE_LENGTH)-1);
 	
	    BitString bitstr = creature1.at(0, pos+1);
	    creature1.at(0,pos+1) = creature2.at(0, pos+1);
	    creature2.at(0,pos+1) = bitstr;
	};
};

void CEvolution_Master::BitFlip_Mutation(
					 MyBitString &creature
					 )
{
    for (int i=0; i<GENOTYPE_LENGTH; i++)
	if (drand48()< REPRODUCE_PROBABILITY_BITFLIP)
	    creature.invert(i);
};

int CEvolution_Master::select_one(
				  float probability[POPULATION_SIZE]
				  )
{
    float rand;
    int i;
    
    do {
	rand = drand48();
	i = 0;
	while ((i<POPULATION_SIZE) && (rand>probability[i++]));
    } while (i >= POPULATION_SIZE);

    // since floats don't have unlimited accurency it is possible that the sum of all probabilities is not excaclty 1 but
    // 0.99999 or so. if now rand would be greater than this, i would not have a good value. simply saying that then it should
    // be the last creature is not fair too. so it is done all over again. This little compare operation should not at all
    // influence overall speed of the program.
    return i;
};

void CEvolution_Master::reproduce()
{
    float probability[POPULATION_SIZE];
    float sum = 0;


    for (int i=0; i<POPULATION_SIZE; i++)
	sum += (probability[i] = pow(1/(REPRODUCE_PROBABILITY_DIVIDE * (i+1 + REPRODUCE_PROBABILITY_ADD)), REPRODUCE_PROBABILITY_EXPONENT) + REPRODUCE_GROUND_PROBABILITY);

    for (int i=0; i<POPULATION_SIZE; i++)
	probability[i] = probability[i] / sum;
    
    for (int i=1; i<POPULATION_SIZE; i++)
	probability[i] += probability[i-1];
    

    MyBitString new_genepool[POPULATION_SIZE];
    int current_pos = 0;
    assert(POPULATION_SIZE % 2 == 0);
    for (int i=0; i< POPULATION_SIZE / 2; i++)
	{
	    MyBitString creature1 = *genepool[select_one(probability)];
	    MyBitString creature2 = *genepool[select_one(probability)];

	    Crossover(creature1, creature2);
	    BitFlip_Mutation(creature1);
	    BitFlip_Mutation(creature2);
	    
	    new_genepool[current_pos++] = creature1;
	    new_genepool[current_pos++] = creature2;
	};
    assert(current_pos = POPULATION_SIZE);

    for (int i=0; i<POPULATION_SIZE; i++)
	*genepool[i] = new_genepool[i];
};

void CEvolution_Master::print_genomes(FILE *out_file)
{
    for (int i=0; i<POPULATION_SIZE; i++)
      {
	for (int j=0; j<GENOTYPE_LENGTH; j++)
	  {
	    if (genepool[i]->test(j))
	      putc('1', out_file);
	    else
	      putc('0', out_file);
	  };
	putc('\n', out_file);
      };
};
