#include <assert.h>

#include "growingaxonlist.h"
#include "growinggrid.h"
#include "growingaxonhead.h"

CGrowing_Axon_List::CGrowing_Axon_List()
{
    next=NULL;
    previous=NULL;

    head=NULL;
};

CGrowing_Axon_List::CGrowing_Axon_List(CGrowing_Axon_List *inext, CGrowing_Axon_List *iprev, CGrowing_Axon_Head *ihead)
{
    assert( (!inext) || (inext->previous==iprev));
    assert( (!iprev) || (iprev->next==inext    ));

    next=inext;
    previous=iprev;

    if (next) next->previous = this;
    if (previous) previous->next = this;
    
    head=ihead;
};

CGrowing_Axon_List::~CGrowing_Axon_List() 
{
    if (previous) // it is not the first in the list
	{
	    assert( (!next)     || (next->previous==this));
	    assert(                (previous->next==this));

	    previous->set_next(next);
	    if (next) next->set_previous(previous);

	    assert( (!next)     || (next->previous==previous));
	    assert(                (previous->next==next    ));

	    if (head) delete head;
	}
    else
	{ // it is the first in the list
	    assert (!head); // should have no head
	    assert( (!next)     || (next->previous==this));
	    while (next) delete next;
	};
};

CGrowing_Axon_List *CGrowing_Axon_List::get_next()
{
    return next;
};

CGrowing_Axon_List *CGrowing_Axon_List::get_previous()
{
    return previous;
};

CGrowing_Axon_Head *CGrowing_Axon_List::get_head()
{
    return head;
};

void CGrowing_Axon_List::set_next(CGrowing_Axon_List *snext)
{
    next=snext;
};

void CGrowing_Axon_List::set_previous(CGrowing_Axon_List *sprevious)
{
    previous=sprevious;
};

void CGrowing_Axon_List::insert(CGrowing_Axon_Head *ihead)
{
    next = new CGrowing_Axon_List(next, this, ihead);    
};

void CGrowing_Axon_List::insert_end(CGrowing_Axon_Head *ihead)
{
    if (!next)
	next = new CGrowing_Axon_List(next, this, ihead);
    else 
	next->insert_end(ihead);
};

bool CGrowing_Axon_List::grow(CGrowing_Grid &grid)
{ 
    int grown = 0;
    CGrowing_Axon_List *nxt = next;

    if (head) grown = head->grow(grid, this);


#ifndef NDEBUG
    // check wheter nxt is still in the list
    CGrowing_Axon_List *t = this->next;
    while (t && (nxt != t)) 
	t = t->next;
    assert( t==nxt );
#endif


    if (grown) {
	if (nxt) nxt->grow(grid);
	if (grown > 0) delete this;
	return true;
    }
    else {
	if (nxt) return (nxt->grow(grid));
	return false;
    };
};
