#include <assert.h>
#include <math.h>

#include "growingcell.h"
#include "growingaxonhead.h"
#include "growingaxonlist.h"

CGrowing_Cell::CGrowing_Cell(
			     C3dpoint ipos,
			     int inumber
			     )
{
    pos=ipos;

    assert(inumber>=0); assert(inumber<NR_OF_CONNECTABLE_CELLS);
    number=inumber;
};

int CGrowing_Cell::get_number() { return number; };



CGrowing_Axon_Cell::CGrowing_Axon_Cell(
				       MyBitString genotype,
				       C3dpoint ipos,
				       int inumber,
				       CGrowing_Axon_List &axonlist
				       ): CGrowing_Cell(ipos, inumber)
{
    int pos=0;
    
    assert(GENOTYPE_LENGTH_AXONGROWING==(NR_OF_SUBSTANCES_TO_FOLLOW * GENOTYPE_LENGTH_SUBSTANCE_TO_FOLLOW +  GENOTYPE_LENGTH_INHIBITORY_PROPABILITY + GENOTYPE_LENGTH_CONNECTIVITY + GENOTYPE_LENGTH_TERMINATE_GROWTH));
    assert(genotype.length()==GENOTYPE_LENGTH_AXONGROWING);
    
    for(int i=0; i<NR_OF_SUBSTANCES_TO_FOLLOW; i++) {
	substances_to_follow[i] = genotype.value_at(pos,GENOTYPE_LENGTH_ONE_SUBSTANCE);
	pos += GENOTYPE_LENGTH_ONE_SUBSTANCE;
	thresholds[i] = genotype.value_at(pos,GENOTYPE_LENGTH_SUBSTANCE_THRESHOLD) / DIVIDE_GENOTYPE_SUBSTANCE_THRESHOLD;
	pos += GENOTYPE_LENGTH_SUBSTANCE_THRESHOLD;
    };
    
    inhibitory_probability = (float)genotype.value_at(pos,GENOTYPE_LENGTH_INHIBITORY_PROPABILITY) / DIVIDE_BITSTRING_INHIBITORY_PROPABILITY;
    pos += GENOTYPE_LENGTH_INHIBITORY_PROPABILITY;
    hairiness = (float)genotype.value_at(pos,GENOTYPE_LENGTH_CONNECTIVITY) / DIVIDE_BITSTRING_CONNECTIVITY;
    pos += GENOTYPE_LENGTH_CONNECTIVITY;

    assert(GENOTYPE_LENGTH_TERMINATE_GROWTH==(GENOTYPE_LENGTH_AXONGROWING-pos));     
    max_inactive_turns = 1+genotype.value_at(pos,GENOTYPE_LENGTH_TERMINATE_GROWTH);
    
    axonlist.insert_end(new CGrowing_Axon_Head(ipos, this, 0, 0));
};

int CGrowing_Axon_Cell::get_number() { return number; };

int CGrowing_Axon_Cell::get_substance_from_substances_to_follow(
								int substance_number
								)
{
  assert(substance_number >= 0); assert(substance_number < NR_OF_SUBSTANCES_TO_FOLLOW);
  return substances_to_follow[substance_number];
};

float CGrowing_Axon_Cell::get_threshold_from_substances_to_follow(
								  int substance_number
								  )
{
  assert(substance_number >= 0); assert(substance_number < NR_OF_SUBSTANCES_TO_FOLLOW);
  return thresholds[substance_number];
};

float CGrowing_Axon_Cell::get_hairiness() { return hairiness; };

bool CGrowing_Axon_Cell::get_inhibitory() { return (drand48() < inhibitory_probability); };

int CGrowing_Axon_Cell::get_max_inactive_turns() {
  return max_inactive_turns;
};





int CGrowing_Dentrite_Cell::get_number() { return number; };

void CGrowing_Dentrite_Cell::get_weight_vector(
					      float weights[NR_OF_CONNECTABLE_CELLS],
					      CGrowing_Axon_List &axon_list
					      )
{
    CGrowing_Axon_List *cur_list = axon_list.get_next();

    assert(cur_list->get_head()!=NULL);
    
    float tmp;
    while (cur_list) {

	weights[cur_list->get_head()->get_home_number()] += 
	    (cur_list->get_head()->get_home()->get_inhibitory() ? -1.0 : 1.0) * 
	    ( ( (tmp = 1.0 - (cur_list->get_head()->get_distance_to_home()*AXON_LENGTH_PENALTY)) < 0 ? 0 : tmp) *
	      1.0/( 
		   (cur_list->get_head()->get_distance_to_position(pos) == 0.0) ?
		   1.0 :
		   pow(cur_list->get_head()->get_distance_to_position(pos),3) * AXON_DISTANCE_PENALTY) 
	      );

	cur_list=cur_list->get_next();
    };

    weights[number]=0;


    float sum_of_weights = 0.0; // = sum ( abs( weight ) );
    for (int i=0; i<NR_OF_CONNECTABLE_CELLS; i++)
	sum_of_weights += fabs(weights[i]);

    float inverse_weight = sum_of_weights == 0.0 ? 0.0 : 1.0 / sum_of_weights;
    for (int i=0; i<NR_OF_CONNECTABLE_CELLS; i++)
	weights[i] = weights[i] * inverse_weight;

};




int CGrowing_Both_Cell::get_number() { return CGrowing_Axon_Cell::number; };
