#ifndef __GROWINGGRID_H
#define __GROWINGGRID_H

#include "constants.h"
#include "MyBitString.h"
#include "3dpoint.h"

// Since Jun27 CGrowing_Grid handles concentration different.
// Previously the concentration array has always been updated when calling add_concentration, so the data
// in grid[][][] was always up-to-date.
//
// now the concentration is only calculated on demand. To make things more simple, _all_ the add_concentration
// have to be called before any get_concentration is be called.

struct TGrowing_Gridcell{
    float concentration[NR_OF_SUBSTANCES];
    bool initialized[NR_OF_SUBSTANCES];
    bool occupied;
};

struct TAxon_Attracting{
    float init_concentration;
    int substance;
    int x;
    int y;
    int z;
};

class CGrowing_Grid {
private:
    TGrowing_Gridcell grid[CUBE_SIZE][CUBE_SIZE][CUBE_SIZE];
    TAxon_Attracting axon_attracting[NR_OF_AXON_ATTRACTING];
    int akt_axon_attracting;

    void add_concentration(
			   C3dpoint pos,
			   float c,
			   int substance
			   );
public:
    CGrowing_Grid();
    ~CGrowing_Grid();

    void add_concentration(
			   MyBitString genotype,
			   C3dpoint pos
			   );
    
    void set_occupied(
		      int x,
		      int y,
		      int z);
    bool get_occupied(
		      int x,
		      int y,
		      int z);

    float get_concentration(
			    int x,
			    int y,
			    int z,
			    int substance
			    );
};










#endif //  __BRAIN_H
