#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <assert.h>

#include "world.h"
// #include "worldviewer.h"

float CWorld::eye(
		  float x,
		  float y,
		  float direction
		  )
{
    int found_food = 0;
    int found_x;
    int found_y;
    
    for(int r=1; r<=EYE_STRENGTH; r++) {
	for(float angle = direction - EYE_WIDTH; angle <= direction + EYE_WIDTH; angle+=(2*PI/360)) {
	    if (((found_y = (int)(y + cos(angle) * r)) >= 0) && (found_y < world_y_size) &&
		((found_x = (int)(x + sin(angle) * r)) >= 0) && (found_x < world_x_size))
	        if ( world[found_y][found_x] == FOOD ) {
		    found_food = r;
		}
	}
	if (found_food) break;
    }
    
    if (found_food) {
	return ( (float)(EYE_STRENGTH - found_food + 1) / (float)EYE_STRENGTH );
    }
    return 0;
}

float CWorld::whisker(
		      int x,
		      int y,
		      float direction
		      )
{
    int cnt = 1;
    int found_x;
    int found_y;
    
    for(int r=1; r<=WHISKER_LENGTH; r++) {
	for(float angle = direction - WHISKER_WIDTH; angle <= direction + WHISKER_WIDTH; angle+=(2*PI/360)) {
	    if (((found_y = (int)(y + cos(angle) * r)) >= 0) && (found_y < world_y_size) &&
		((found_x = (int)(x + sin(angle) * r)) >= 0) && (found_x < world_x_size)) {
		if ( world[found_y][found_x] == OBSTACLE ) {
		    cnt++;
		}
	    } else
		cnt++;
	}
    }
//    printf("%i\n",cnt);
    return ( 1.0 / (float)cnt );
}


CWorld::CWorld(
	       const char *map_name
	       )
{
    world_x_size= 0;
    world_y_size= 0;
    read_new_map(map_name);
    world = (int **) calloc(world_y_size, sizeof(int *));
    for (int i=0; i<world_y_size; i++)
	world[i] = (int *) calloc(world_x_size, sizeof(int));
}

CWorld::~CWorld()
{
    for (int i=0; i<world_y_size; i++)
	free(empty_world[i]);
    free(empty_world);
}

void CWorld::read_new_map(
			  const char *map_name
			  ) 
{
    FILE *map_file;
    char c;
    
    map_file = fopen(map_name,"r");
    if (map_file != NULL) {
	
	for (int i=0; i<world_y_size; i++)
	    free(empty_world[i]);
	if (world_y_size) free(empty_world);
	
	world_x_size= 0;
	c= '0';
	do {
	    world_x_size *= 10;
	    world_x_size += (c-'0');
	    c = fgetc(map_file);
	} while (c != '\n');
	world_y_size= 0;
	c= '0';
	do {
	    world_y_size *= 10;
	    world_y_size += (c-'0');
	    c = fgetc(map_file);
	} while (c != '\n');

	empty_world = (int **) calloc(world_y_size, sizeof(int *));
	for (int i=0; i<world_y_size; i++)
	    empty_world[i] = (int *) calloc(world_x_size, sizeof(int));
	
	for (int i=0; i<world_y_size; i++) {
	    for (int j=0; j<world_x_size; j++) {
		c = fgetc(map_file);
		if (c == 'X')
		    empty_world[i][j] = OBSTACLE;
		else
		    empty_world[i][j] = EMPTY;
	    }
 	    do
 		c = fgetc(map_file);
 	    while ((c != '\n') && (c != EOF));
	}
	
	fclose(map_file);
    };
};


void CWorld::evaluate(MyBitString *genotypes[POPULATION_SIZE])
{
    assert(world_y_size);
    assert(world_x_size);

    world = (int **) calloc(world_y_size, sizeof(int *));
    for (int i=0; i<world_y_size; i++)
	world[i] = (int *) calloc(world_x_size, sizeof(int));
    for(int i=0; i<world_y_size; i++)
	for( int j=0; j<world_x_size; j++)
	    world[i][j]= (empty_world[i][j] == OBSTACLE ? OBSTACLE : (FOOD_PROBABILITY > drand48() ? FOOD : EMPTY));

    CCreature *creatures[POPULATION_SIZE];
    
    {
	int x;
	int y;
	for (int i=0; i<POPULATION_SIZE; i++) {
	    do {
		x = (int)(drand48() * (world_x_size - 1));
		y = (int)(drand48() * (world_y_size - 1));
	    } while (world[y][x] == OBSTACLE);

	    creatures[i] = new CCreature(*genotypes[i], x, y, drand48()*2*PI);
printf("%i\n",i);

	    if (world[y][x] == FOOD) {
		creatures[i]-> eat();
		world[y][x] = EMPTY;
	    }
	}
    }

    int pos= 0;
    float dir;
    float x, old_x;
    float y, old_y;
    float sensors[NR_OF_SENSORS];
    MyBitString ordered_genotypes[POPULATION_SIZE];

    //    CWorld_Viewer *view = new CWorld_Viewer();
  
    while (pos < POPULATION_SIZE) {
	for(int i=0; i<POPULATION_SIZE; i++) {
	    if (creatures[i]-> alive()) {
		x= old_x= creatures[i]-> get_x();
		y= old_y= creatures[i]-> get_y();
		dir= creatures[i]-> get_direction() + EYE_START - EYE_STEP;
		for(int j=0; j<NR_OF_EYES; j++) sensors[j]= eye(x,y,dir+=EYE_STEP);
		dir= creatures[i]-> get_direction() + WHISKER_START - WHISKER_STEP;
		for(int j=NR_OF_EYES; j<NR_OF_WHISKERS+NR_OF_EYES; j++) sensors[j]= whisker(x,y,dir+=WHISKER_STEP);

		//for(int j=0; j<NR_OF_WHISKERS+NR_OF_EYES; j++) printf("%f    ",sensors[j]);
		//printf("\n");
		
		creatures[i]-> move(sensors);
		x= creatures[i]-> get_x();
		y= creatures[i]-> get_y();
		dir= creatures[i]-> get_direction();

		if (int(y)<0 || int(y) >= world_y_size ||
		    int(x)<0 || int(x) >= world_x_size ||
		    (world[(int)y][(int)x] == OBSTACLE)) {
		    creatures[i]-> crash();
		    creatures[i]-> set_x(old_x);
		    creatures[i]-> set_y(old_y);
		    creatures[i]-> set_direction(dir + ( (drand48() - 0.5) * 90) );
		}
		else
		    if (world[(int)y][(int)x] == FOOD)
			{
			    creatures[i]-> eat();
			    world[(int)y][(int)x] = EMPTY;
			};
			    
		
		creatures[i]-> live_cycle();

		if (!creatures[i]-> alive())
		    {
		        ordered_genotypes[pos++] = *genotypes[i];
printf("und tot\n");
		    };
	    };
	};

	//	 view->show(world_x_size, world_y_size, world, creatures);

    };

    //delete view;

    for(int i=0; i<POPULATION_SIZE; i++)
	delete creatures[i];

    for(int i=0; i<POPULATION_SIZE; i++)
      *genotypes[i] = ordered_genotypes[POPULATION_SIZE-i-1];

    for (int i=0; i<world_y_size; i++)
	free(world[i]);
    if (world_y_size) free(world);
}
